#' Scorecards skill score calculation
#' 
#'@description Scorecards function to calculate the skill scores from the 
#'  spacially aggregated scores.
#'
#'@param aggregated_metrics is a list of arrays containing the spatially aggregated metrics
#'  by the function SC_spatial_aggregation.
#'@param metrics is a vector of the metrics to by aggregated, it should be the 
#'  same vector as introduced in the function SC_spatial_aggregation.
#'
#'@example
#'skill_scores <- SC_skill_scores(data = aggregated_metrics,
#'                                metrics = c('mean_bias', 'enscorr', 'rps', 'rps_clim', 'crps', 'crps_clim', 'enssprerr'),
#'                                ncores = 8)


SC_skill_scores <- function(data,
                            metrics, 
                            ncores){


  
## For testing:
  data <- aggregated_metrics
  metrics <- c('mean_bias', 'enscorr', 'rps', 'rps_clim', 'crps', 'crps_clim', 'enssprerr')
  ncores <- 8
  
  
## TO DO: checks
## Check that rps, rps_clim, crps and crps_clim are present in data 
  
  
  ## Make sure dimensions are in correct order
  aggregated_metrics <- s2dv::Reorder(data = aggregated_metrics, order = c('system','reference','metric','time','sdate','region'))
  
  
  
  ## calculate rpss from rps and rps_clim and replace rpss values with newly calculated values

  # aggregated_metrics_clim <- s2dv::Reorder(data = aggregated_metrics_clim, order = c('system','reference','metric','time','sdate','region'))
  aggregated_metrics[,,which(metrics=='rpss'),,,] <- multiApply::Apply(data = list(rps = aggregated_metrics[,,which(metrics=='rps'),,,],
                                                                              rps_clim = aggregated_metrics_clim[,,which(metrics=='rps_clim'),,,]),
                                                                  target_dims = NULL,
                                                                  fun = function(rps,rps_clim){1 - rps/rps_clim},
                                                                  ncores = ncores)$output1
  
  ## for creating new rpss metric without replacing current one, need to use abind()
  
  ## calculate crpss from crps and crps_clim and replace crpss values with newly calculated values
  
  aggregated_metrics[,,which(metrics=='crpss'),,,] <- multiApply::Apply(data = list(crps = aggregated_metrics[,,which(metrics=='crps'),,,],
                                                                                   crps_clim = aggregated_metrics_clim[,,which(metrics=='crps_clim'),,,]),
                                                                       target_dims = NULL,
                                                                       fun = function(crps,crps_clim){1 - crps/crps_clim},
                                                                       ncores = ncores)$output1
  
  
} ## close function
